__all__ = ['Punishment']

import datetime
from dataclasses import dataclass
from typing import Callable

from multibot.models import Platform, PunishmentBase, db

from flanabot import constants


@dataclass(eq=False)
class Punishment(PunishmentBase):
    collection = db.punishment

    @classmethod
    async def check_olds(cls, unpunishment_method: Callable, platform: Platform):
        punishment_groups = cls._get_grouped_punishments(platform)

        now = datetime.datetime.now(datetime.timezone.utc)
        for (_, _), sorted_punishments in punishment_groups:
            if not (last_punishment := sorted_punishments[-1]).until or now < last_punishment.until:
                continue

            if last_punishment.until + constants.PUNISHMENTS_RESET <= now:
                for old_punishment in sorted_punishments:
                    old_punishment.delete()

            if last_punishment.is_active:
                await last_punishment.unpunish(unpunishment_method)
                last_punishment.is_active = False
                last_punishment.save()



from __future__ import annotations  # todo0 remove in 3.11

__all__ = ['Message']

from dataclasses import dataclass, field
from typing import Iterable

from flanautils import Media, OrderedSet
from multibot.models import Message as MultiBotMessage, User

from flanabot.models.chat import Chat
from flanabot.models.weather_chart import WeatherChart


@dataclass(eq=False)
class Message(MultiBotMessage):
    author: User = None
    button_pressed_user: User = None
    mentions: Iterable[User] = field(default_factory=list)
    chat: Chat = None
    replied_message: Message = None
    weather_chart: WeatherChart = None
    song_infos: OrderedSet[Media] = field(default_factory=OrderedSet)



import datetime

import flanautils

AUTO_WEATHER_EVERY = datetime.timedelta(hours=6)
CHECK_PUNISHMENTS_EVERY_SECONDS = datetime.timedelta(hours=1).total_seconds()
HEAT_PERIOD_SECONDS = datetime.timedelta(minutes=15).total_seconds()
INSULT_PROBABILITY = 0.00166666667
MAX_PLACE_QUERY_LENGTH = 50
PUNISHMENT_INCREMENT_EXPONENT = 6
PUNISHMENTS_RESET = datetime.timedelta(weeks=6 * flanautils.WEEKS_IN_A_MONTH)
RECOVERY_DELETED_MESSAGE_BEFORE = datetime.timedelta(hours=1)
SCRAPING_MESSAGE_WAITING_TIME = 0.1

BYE_PHRASES = ('Adiós.', 'adieu', 'adio', 'adioh', 'adios', 'adió', 'adiós', 'agur', 'bye', 'byyeeee', 'chao',
               'hasta la vista', 'hasta luego', 'hasta nunca', ' hasta pronto', 'hasta la próxima',
               'nos vemos', 'taluego')

HELLO_PHRASES = ('alo', 'aloh', 'buenas', 'Hola.', 'hello', 'hey', 'hi', 'hola', 'holaaaa', 'holaaaaaaa', 'ola',
                 'ola k ase', 'pa ti mi cola', 'saludos')
INSULTS = (
    'Cállate ya anda.',
    '¿Quién te ha preguntado?',
    '¿Tú eres así o te dan apagones cerebrales?',
    'Ante la duda mi dedo corazón te saluda.',
    'Enjoy cancer brain.',
    'Calla noob.',
    'Hablas tanta mierda que tu culo tiene envidia de tu boca.',
    'jAJjajAJjajAJjajAJajJAJajJA',
    'enjoy xd',
    'Reported.',
    'Baneito pa ti en breve.',
    'Despídete de tu cuenta.',
    'Flanagan es más guapo que tú.',
    'jajaj',
    'xd',
    'Hay un concurso de hostias y tienes todas las papeletas.',
    '¿Por qué no te callas?',
    'Das penilla.',
    'Deberían hacerte la táctica del C4.',
    'Te voy romper las pelotas.',
    'Más tonto y no naces.',
    'Eres más tonto que peinar bombillas.',
    'Eres más tonto que pellizcar cristales.',
    'Eres más malo que pegarle a un padre.'
)

KEYWORDS = {
    'covid_chart': ('case', 'caso', 'contagiado', 'contagio', 'corona', 'coronavirus', 'covid', 'covid19', 'death',
                    'disease', 'enfermedad', 'enfermos', 'fallecido', 'incidencia', 'jacovid', 'mascarilla', 'muerte',
                    'muerto', 'pandemia', 'sick', 'virus'),
    'currency_chart': ('argentina', 'bitcoin', 'cardano', 'cripto', 'crypto', 'criptodivisa', 'cryptodivisa',
                       'cryptomoneda', 'cryptocurrency', 'currency', 'dinero', 'divisa', 'ethereum', 'inversion',
                       'moneda', 'pasta'),
    'punish': ('acaba', 'aprende', 'ataca', 'atalo', 'azota', 'boss', 'castiga', 'castigo', 'condena', 'controla',
               'destroy', 'destroza', 'duro', 'ejecuta', 'enseña', 'escarmiento', 'execute', 'finish', 'fuck', 'fusila',
               'hell', 'humos', 'infierno', 'jefe', 'jode', 'learn', 'leccion', 'lesson', 'manda', 'purgatorio',
               'sancion', 'shoot', 'teach', 'termina', 'whip'),
    'scraping': ('api', 'aqui', 'busca', 'contenido', 'content', 'descarga', 'descargar', 'download', 'envia', 'habia',
                 'media', 'redes', 'scrap', 'scraping', 'search', 'send', 'social', 'sociales', 'tenia', 'video',
                 'videos'),
    'song_info': ('aqui', 'cancion', 'data', 'datos', 'info', 'informacion', 'information', 'llama', 'media', 'name',
                  'nombre', 'sonaba', 'sonando', 'song', 'sono', 'sound', 'suena', 'title', 'titulo',
                  'video'),
    'unpunish': ('absolve', 'forgive', 'innocent', 'inocente', 'perdona', 'spare'),
    'weather_chart': ('atmosfera', 'atmosferico', 'calle', 'calor', 'caloret', 'clima', 'climatologia', 'cloud',
                      'cloudless', 'cloudy', 'cold', 'congelar', 'congelado', 'denbora', 'despejado', 'diluvio', 'frio',
                      'frost', 'hielo', 'humedad', 'llover', 'llueva', 'llueve', 'lluvia', 'nevada', 'nieva', 'nieve',
                      'nube', 'nubes', 'nublado', 'meteorologia', 'rain', 'snow', 'snowfall', 'snowstorm', 'sol',
                      'solano', 'storm', 'sun', 'temperatura', 'tempo', 'tiempo', 'tormenta', 'ventisca', 'weather',
                      'wetter')
}

SCRAPING_PHRASES = ('Analizando...', 'Buscando...', 'Hackeando internet... 👀', 'Rebuscando en la web...',
                    'Robando cosas...', 'Scrapeando...', 'Scraping...')
